"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadRspackConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const findConfig_1 = __importDefault(require("./findConfig"));
const rechoir_1 = __importDefault(require("rechoir"));
const interpret_1 = __importDefault(require("interpret"));
const isEsmFile_1 = __importDefault(require("./isEsmFile"));
const isTsFile_1 = __importDefault(require("./isTsFile"));
const crossImport_1 = __importDefault(require("./crossImport"));
const DEFAULT_CONFIG_NAME = "rspack.config";
const registerLoader = (configPath) => {
    const ext = path_1.default.extname(configPath);
    // TODO implement good `.mts` support after https://github.com/gulpjs/rechoir/issues/43
    // For ESM and `.mts` you need to use: 'NODE_OPTIONS="--loader ts-node/esm" rspack build --config ./rspack.config.mts'
    if ((0, isEsmFile_1.default)(configPath) && (0, isTsFile_1.default)(configPath)) {
        return;
    }
    const extensions = Object.fromEntries(Object.entries(interpret_1.default.extensions).filter(([key]) => key === ext));
    if (Object.keys(extensions).length === 0) {
        throw new Error(`config file "${configPath}" is not supported.`);
    }
    try {
        rechoir_1.default.prepare(extensions, configPath);
    }
    catch (error) {
        const failures = error === null || error === void 0 ? void 0 : error.failures;
        if (failures) {
            const messages = failures.map(failure => failure.error.message);
            throw new Error(`${messages.join("\n")}`);
        }
        else {
            throw error;
        }
    }
};
async function loadRspackConfig(options, cwd = process.cwd()) {
    if (options.config) {
        const configPath = path_1.default.resolve(cwd, options.config);
        if (!fs_1.default.existsSync(configPath)) {
            throw new Error(`config file "${configPath}" not found.`);
        }
        (0, isTsFile_1.default)(configPath) && registerLoader(configPath);
        return (0, crossImport_1.default)(configPath, cwd);
    }
    else {
        const defaultConfig = (0, findConfig_1.default)(path_1.default.resolve(cwd, DEFAULT_CONFIG_NAME));
        if (defaultConfig) {
            (0, isTsFile_1.default)(defaultConfig) && registerLoader(defaultConfig);
            return (0, crossImport_1.default)(defaultConfig, cwd);
        }
        else {
            return {};
        }
    }
}
exports.loadRspackConfig = loadRspackConfig;
